dnl Macros for configuring WHIZARD subdirectories and subpackages
dnl All macros begin with 'WK'

dnl Message at the beginning of a configure section
AC_DEFUN(WK_CONFIGURE_SECTION,
[AC_MSG_NOTICE([--------------------------------------------------------------])
AC_MSG_NOTICE([--- ]$1[ ---])
AC_MSG_NOTICE([])
])


dnl Define a variable and export it
dnl WK_WHIZARD_SET(variable, value)
AC_DEFUN(WK_WHIZARD_SET,
[$1=$2
AC_SUBST($1)])

dnl Add a list of names to the list of output resp. executable files
dnl WK_WHIZARD_OUTFILES(subdir, files)
AC_DEFUN(WK_WHIZARD_OUTFILES,
[for file in $2; do
  OUTFILES="$OUTFILES $1/$file";
done])

dnl WK_WHIZARD_EXECUTABLES(subdir, files)
AC_DEFUN(WK_WHIZARD_EXECUTABLES,
[for file in $2; do
  OUTFILES="$OUTFILES $1/$file";
  EXECUTABLES="$EXECUTABLES $1/$file"
done])


dnl This adds a package which resides in a WHIZARD subdirectory.
dnl The `variable' is inserted into AC_SUBST; it will refer to 
enl the package subdirectory in Makefiles.  The package
dnl has a subdirectory XXX-src and optionally a Makefile (or similar).
dnl WK_WHIZARD_PACKAGE(variable, identifier [,Makefiles [,Executables]])
AC_DEFUN(WK_WHIZARD_PACKAGE,
[WK_WHIZARD_SET($1,$2-src)
ifelse($#, 3, 
[WK_WHIZARD_OUTFILES($$1, $3)])
ifelse($#, 4, 
[WK_WHIZARD_OUTFILES($$1, $3)
WK_WHIZARD_EXECUTABLES($$1, $4)])
])

dnl This is similar, but no '-src' is appended for the directory name
dnl WK_WHIZARD_SUBDIR(variable, identifier [,Makefiles [,Executables]])
AC_DEFUN(WK_WHIZARD_SUBDIR,
[WK_WHIZARD_SET($1,$2)
ifelse($#, 3,
[WK_WHIZARD_OUTFILES($$1, $3)])
ifelse($#, 4, 
[WK_WHIZARD_OUTFILES($$1, $3)
WK_WHIZARD_EXECUTABLES($$1, $4)])
])


dnl This adds a matrix element generating package.
dnl There must be at least one Makefile.
dnl In addition to the preceding macro, it adds two scripts
dnl init.XXX and make-proc.XXX where XXX is the package identifier.
dnl WK_WHIZARD_ME_GENERATOR(variable, identifier, Makefile)
AC_DEFUN(WK_WHIZARD_ME_GENERATOR,
[WK_WHIZARD_PACKAGE($1,$2,$3, init.$2 make-proc.$2)])


dnl This is like WK_WHIZARD_PACKAGE, but it calls AC_ARG_ENABLE in addition.
dnl If the package `id' is disabled or the 'file' is not found,
dnl the variable `var' is set to "no".
dnl WK_WHIZARD_PACKAGE_ENABLE(var, id, help, file [,Makefiles [,Executables]])
AC_DEFUN(WK_WHIZARD_PACKAGE_ENABLE,
[AC_ARG_ENABLE($2,[$3])
if test "$enable_$2" = "no"; then
AC_MSG_CHECKING([for $2-src/$4])
WK_WHIZARD_SET($1, no)
AC_MSG_RESULT([(disabled)])
else
AC_CHECK_PROG(enable_$2, $2-src/$4, yes, no, .)
if test "$enable_$2" = "no"; then
WK_WHIZARD_SET($1, no)
else
ifelse($#, 4, [WK_WHIZARD_PACKAGE($1, $2)])
ifelse($#, 5, [WK_WHIZARD_PACKAGE($1, $2, $5)])
ifelse($#, 6, [WK_WHIZARD_PACKAGE($1, $2, $5, $6)])
fi
fi
])

dnl The same, but disabled by default.
dnl If the package `id' is disabled or the 'file' is not found,
dnl the variable `var' is set to "no".
dnl WK_WHIZARD_PACKAGE_DISABLE(var, id, help, file [,Makefiles [,Executables]])
AC_DEFUN(WK_WHIZARD_PACKAGE_DISABLE,
[AC_ARG_ENABLE($2,[$3])
if test "$enable_$2" = "yes"; then
AC_CHECK_PROG(enable_$2, $2-src/$4, yes, no, .)
if test "$enable_$2" = "no"; then
WK_WHIZARD_SET($1, no)
else
ifelse($#, 4, [WK_WHIZARD_PACKAGE($1, $2)])
ifelse($#, 5, [WK_WHIZARD_PACKAGE($1, $2, $5)])
ifelse($#, 6, [WK_WHIZARD_PACKAGE($1, $2, $5, $6)])
fi
else
AC_MSG_CHECKING([for $2-src/$4])
WK_WHIZARD_SET($1, no)
AC_MSG_RESULT([(disabled)])
fi
])

dnl The same for a matrix element generator
dnl There must be at least one Makefile.
dnl WK_WHIZARD_ME_GENERATOR_ENABLE(var, id, help, file, Makefiles [,Executables])
AC_DEFUN(WK_WHIZARD_ME_GENERATOR_ENABLE,
[WK_WHIZARD_PACKAGE_ENABLE($1, $2, [$3], $4, $5, init.$2 make-proc.$2 $5)])

dnl The same, disabled by default:
dnl WK_WHIZARD_ME_GENERATOR_DISABLE(var, id, help, file, Makefiles [,Executables])
AC_DEFUN(WK_WHIZARD_ME_GENERATOR_DISABLE,
[WK_WHIZARD_PACKAGE_DISABLE($1, $2, [$3], $4, $5, init.$2 make-proc.$2 $5)])



dnl Extension of AC_PATH_PROG: Search for libraries for which the name
dnl is not exactly known (because it may have the version number in it)
dnl Set output variables $var, $var_DIR, $var_LIB accordingly
dnl WHIZARD_PATH_LIB(var, id, name-list, path)
AC_DEFUN(WHIZARD_PATH_LIB,
[AC_CACHE_CHECK(for $3, wk_cv_path_$1,
[case "$$1" in
  /*)
  wk_cv_path_$1="$$1" # User-supplied path
  ;;
  *)
  IFS="${IFS= 	}"; ac_save_ifs="$IFS"; IFS=":"
  ac_dummy=$4
  for ac_dir in $ac_dummy; do
    test -z "$ac_dir" && ac_dir=.
    unset wk_cv_path_$1
    ac_pwd=`pwd`
    if test -d "$ac_dir"; then
      cd $ac_dir
      for ac_word in $3; do
        test -f "$ac_word" && wk_cv_path_$1="$ac_dir/$ac_word"
      done
      cd $ac_pwd
    fi
    if test -n "$wk_cv_path_$1"; then
      break
    fi
  done
  IFS="$ac_save_ifs"
  if test -z "$wk_cv_path_$1"; then
    wk_cv_path_$1="no"
  fi
  ;;
esac
])
$1=$wk_cv_path_$1
if test "$$1" != "no"; then
$1_DIR=`echo $$1 | sed -e 's|/lib$2.*\.a$||'`
$1_LIB=`echo $$1 | sed -e 's|^.*/lib\($2.*\)\.a$|\1|'`
fi
AC_SUBST($1)
AC_SUBST($1_DIR)
AC_SUBST($1_LIB)
])


dnl Library search with enable option
dnl WHIZARD_LIB_ENABLE(var, id, help, name-list, path)
AC_DEFUN(WHIZARD_LIB_ENABLE,
[AC_ARG_ENABLE($2,[$3])
if test "$enable_$2" = "no"; then
AC_MSG_CHECKING(for $4)
$1=no
AC_MSG_RESULT([(disabled)])
else
WHIZARD_PATH_LIB($1, $2, $4, $5)
fi
])

dnl Library search with enable option, disabled by default
dnl WHIZARD_LIB_DISABLE(var, id, help, name-list, path)
AC_DEFUN(WHIZARD_LIB_DISABLE,
[AC_ARG_ENABLE($2,[$3])
if test "$enable_$2" = "yes"; then
WHIZARD_PATH_LIB($1, $2, $4, $5)
else
AC_MSG_CHECKING(for $4)
$1=no
AC_MSG_RESULT([(disabled)])
fi
])


dnl ------------------------------------------------------------------------
dnl Compiler Tests

dnl These are slightly modified version of the default test for f77

dnl Determine a Fortran 90 compiler to use.  If `F95' or `F90' is not set,
dnl check for `lf95', `f95', `f90', `ifc' and `pgf90', in that order.
dnl lf95 must be tried before f95, since Lahey includes
dnl a bogus f95 executable ...
dnl Set the output variable `F90' to the name of the compiler found.
dnl 
dnl WK_PROG_F90()
AC_DEFUN(WK_PROG_F90,
[AC_BEFORE([$0], [AC_PROG_CPP])dnl
test -n "$F95" && F90=$F95
if test -z "$F90"; then
AC_PATH_PROGS(F90, lf95 f95 f90 ifort g95 pgf90)
  test -z "$F90" && AC_MSG_ERROR([no acceptable Fortran 90 compiler found in \$PATH])
fi
WK_PROG_F90_WORKS
])

dnl Test whether the Fortran 90 compiler can compile and link a trivial
dnl Fortran program.  
dnl Disabled: Also, test whether the Fortran 90 compiler is a cross-compiler.
dnl 
dnl WK_PROG_F90_WORKS()
AC_DEFUN(WK_PROG_F90_WORKS,
[test -n "$F95FLAGS" && F90FLAGS=$F95FLAGS
AC_CACHE_CHECK([whether the Fortran compiler ($F90 $F90FLAGS $LDFLAGS) works],
[wk_cv_prog_f90_works],
[AC_LANG_SAVE
WK_LANG_FORTRAN90
WK_TRY_COMPILER_NOINCLUDE(dnl
[program conftest
end program conftest
], wk_cv_prog_f90_works, wk_cv_prog_f90_cross)
AC_LANG_RESTORE
])

if test $wk_cv_prog_f90_works = no; then
  AC_MSG_ERROR([installation or configuration problem: Fortran 90 compiler cannot create executables.])
fi

dnl This value is either in the cache or has been determined above.
dnl AC_CACHE_CHECK([whether this compiler is a cross-compiler],
dnl [wk_cv_prog_f90_cross],
dnl [:])

AC_CACHE_CHECK([the compiler vendor],
[wk_cv_prog_f90_type],
[$F90 -version >conftest.log 2>&1
$F90 -V >>conftest.log 2>&1
$F90 --version >>conftest.log 2>&1
 
wk_grep_f90_NAG=`grep NAGWare conftest.log | head -1`
wk_grep_f90_Compaq=`grep Compaq conftest.log | head -1`
wk_grep_f90_Digital=`grep DIGITAL conftest.log | head -1`
wk_grep_f90_SGI=`grep MIPS conftest.log | head -1`
wk_grep_f90_Intel=`grep 'Intel(R)' conftest.log | head -1`
wk_grep_f90_Lahey=`grep 'Lahey' conftest.log | head -1`
wk_grep_f90_PGI=`grep 'pgf90 ' conftest.log | head -1`
wk_grep_f90_GNU=`grep 'g95 (GCC)' conftest.log | head -1`
 
if test -n "$wk_grep_f90_NAG"; then
  wk_cv_prog_f90_type="NAG"
  wk_cv_prog_f90_version=$wk_grep_f90_NAG
  wk_cv_prog_f90_major_version=[`echo $wk_cv_prog_f90_version | sed -e 's/.*Release \([0-9][0-9]*\)\..*/\1/'`]
elif test -n "$wk_grep_f90_Compaq"; then
  wk_cv_prog_f90_type="Compaq"
  wk_cv_prog_f90_version=$wk_grep_f90_Compaq
elif test -n "$wk_grep_f90_Digital"; then
  wk_cv_prog_f90_type="DEC"
  wk_cv_prog_f90_version=$wk_grep_f90_Digital
elif test -n "$wk_grep_f90_SGI"; then
  wk_cv_prog_f90_type="SGI"
  wk_cv_prog_f90_version=$wk_grep_f90_SGI
elif test -n "$wk_grep_f90_Intel"; then
  wk_cv_prog_f90_type="Intel"
  wk_cv_prog_f90_version=$wk_grep_f90_Intel
  wk_cv_prog_f90_major_version=[`echo $wk_cv_prog_f90_version | sed -e 's/.*Version \([0-9][0-9]*\)\..*/\1/'`]
elif test -n "$wk_grep_f90_Lahey"; then
  wk_cv_prog_f90_type="Lahey"
  wk_cv_prog_f90_version=$wk_grep_f90_Lahey
elif test -n "$wk_grep_f90_PGI"; then
  wk_cv_prog_f90_type="PGI"
  wk_cv_prog_f90_version=$wk_grep_f90_PGI
elif test -n "$wk_grep_f90_GNU"; then
  wk_cv_prog_f90_type="GNU"
  wk_cv_prog_f90_version=$wk_grep_f90_GNU
else
  wk_cv_prog_f90_type="unknown"
  wk_cv_prog_f90_version="unknown"
fi

rm -f conftest.log
])

dnl If this is not in the cache, it has been set above
AC_CACHE_CHECK([the compiler version],
[wk_cv_prog_f90_version], 
[:])

AC_CACHE_CHECK([whether this compiler permits quadruple precision],
[wk_cv_prog_f90_quadra],
[AC_LANG_SAVE
WK_LANG_FORTRAN90
WK_TRY_COMPILER_NOINCLUDE(dnl
[program conftest
integer, parameter :: d=selected_real_kind(precision(1.)+1, range(1.)+1)
integer, parameter :: q=selected_real_kind(precision(1._d)+1, range(1._d))
real(kind=q) :: x
end program conftest
], wk_cv_prog_f90_quadra, wk_cv_prog_f90_cross)
AC_LANG_RESTORE
])

AC_CACHE_CHECK([whether this compiler permits quadruple complex],
[wk_cv_prog_f90_qcomplex],
[AC_LANG_SAVE
WK_LANG_FORTRAN90
WK_TRY_COMPILER_NOINCLUDE(dnl
[program conftest
integer, parameter :: d=selected_real_kind(precision(1.)+1, range(1.)+1)
integer, parameter :: q=selected_real_kind(precision(1._d)+1, range(1._d))
complex(kind=q) :: x
end program conftest
], wk_cv_prog_f90_qcomplex, wk_cv_prog_f90_qcross)
AC_LANG_RESTORE
])

if test "$F90PURE" != "no"; then
AC_CACHE_CHECK([whether this compiler accepts PURE functions],
[wk_cv_prog_f90_pure],
[AC_LANG_SAVE
WK_LANG_FORTRAN90
WK_TRY_COMPILER_NOINCLUDE(dnl
[program conftest
contains
  pure function f(x)
    real, intent(in) :: x
    real :: f
    f = x
  end function f
end program conftest
], wk_cv_prog_f90_pure, wk_cv_prog_f90_cross)
AC_LANG_RESTORE
])
else
AC_MSG_CHECKING([whether this compiler accepts PURE functions])
wk_cv_prog_f90_pure="no"
AC_MSG_RESULT([(disabled)])
fi
])


dnl WK_LANG_FORTRAN90()
AC_DEFUN(WK_LANG_FORTRAN90,
[define([AC_LANG], [FORTRAN90])dnl
ac_ext=f90
ac_compile='${F90-f90} -c $F90FLAGS conftest.$ac_ext 1>&AC_FD_CC'
ac_link='${F90-f90} -o conftest${ac_exeext} $F90FLAGS $LDFLAGS conftest.$ac_ext $LIBS 1>&AC_FD_CC'
cross_compiling=$wk_cv_prog_f90_cross
])

dnl Try to compile, link and execute TEST-PROGRAM.  Set WORKING-VAR to
dnl `yes' if the current compiler works, otherwise set it ti `no'.  Set
dnl CROSS-VAR to `yes' if the compiler and linker produce non-native
dnl executables, otherwise set it to `no'.  Before calling
dnl `AC_TRY_COMPILER()', call `AC_LANG_*' to set-up for the right
dnl language.
dnl
dnl Changes: do not include an #include line
dnl 
dnl WK_TRY_COMPILER_NOINCLUDE(TEST-PROGRAM, WORKING-VAR, CROSS-VAR)
AC_DEFUN(WK_TRY_COMPILER_NOINCLUDE,
[cat > conftest.$ac_ext << EOF
[$1]
EOF
if AC_TRY_EVAL(ac_link) && test -s conftest${ac_exeext}; then
  [$2]=yes
  # If we can't run a trivial program, we are probably using a cross compiler.
  if (./conftest; exit) 2>/dev/null; then
    [$3]=no
  else
    [$3]=yes
  fi
else
  echo "configure: failed program was:" >&AC_FD_CC
  cat conftest.$ac_ext >&AC_FD_CC
  [$2]=no
fi
rm -fr conftest*])



dnl Test for GNU tar:
dnl
dnl WK_PROG_GNU_TAR()
AC_DEFUN(WK_PROG_GNU_TAR,
[AC_CACHE_CHECK([for GNU tar], [wk_cv_prog_GTAR],
[wk_cv_prog_GTAR=$GTAR
test -z "$wk_cv_prog_GTAR" && wk_cv_prog_GTAR=tar
if test -n "`$wk_cv_prog_GTAR --version 2>/dev/null | grep GNU`"; then
  :
elif test -n "`gtar --version 2>/dev/null | grep GNU`"; then
  wk_cv_prog_GTAR=gtar
else
  wk_cv_prog_GTAR=no
fi
])
GTAR=$wk_cv_prog_GTAR
])

dnl Test for GNU make:
dnl
dnl WK_PROG_GNU_MAKE()
AC_DEFUN(WK_PROG_GNU_MAKE,
[AC_CACHE_CHECK([for GNU make], [wk_cv_prog_GMAKE],
[wk_cv_prog_GMAKE="$GMAKE"  # User-supplied command
test -z "$wk_cv_prog_GMAKE" && wk_cv_prog_GMAKE="make"
if test -n "`$wk_cv_prog_GMAKE --version 2>/dev/null | grep GNU`"; then
  :
elif test -n "`gmake --version 2>/dev/null | grep GNU`"; then
  wk_cv_prog_GMAKE=gmake
else
  wk_cv_prog_GMAKE=no
fi
])
GMAKE=$wk_cv_prog_GMAKE
])

dnl Special macros:

dnl Test whether the C compiler as specified in the CompHEP settings 
dnl can compile and link a trivial C program.
dnl 
dnl WK_PROG_COMPHEP_CC_WORKS()
AC_DEFUN(WK_PROG_COMPHEP_CC_WORKS,
[AC_CACHE_CHECK([whether the C compiler selected by CompHEP works], [wk_cv_prog_COMPHEP_CC_works],
[AC_LANG_SAVE
WK_LANG_COMPHEP_CC
WK_TRY_COMPILER_NOINCLUDE([main(){return(0);}], wk_cv_prog_COMPHEP_CC_works, wk_cv_prog_COMPHEP_CC_cross)
AC_LANG_RESTORE
])
if test $wk_cv_prog_COMPHEP_CC_works = no; then
  AC_MSG_ERROR([installation or configuration problem: C compiler cannot create executables.
Check $comphepcom/CC if it is set up correctly])
fi

dnl AC_CACHE_CHECK([whether this compiler is a cross-compiler],
dnl [wk_cv_prog_COMPHEP_CC_cross],
dnl [cross_compiling=$wk_cv_prog_COMPHEP_CC_cross
dnl ])
])

dnl Test whether the F77 compiler as specified in the CompHEP settings 
dnl can compile and link a trivial program.
dnl 
dnl WK_PROG_COMPHEP_F77_WORKS()
AC_DEFUN(WK_PROG_COMPHEP_F77_WORKS,
[AC_CACHE_CHECK([whether the Fortran77 compiler selected by CompHEP works],
[wk_cv_prog_COMPHEP_F77_works],
[AC_LANG_SAVE
WK_LANG_COMPHEP_F77
WK_TRY_COMPILER_NOINCLUDE(dnl
[      program conftest
      end
], wk_cv_prog_COMPHEP_F77_works, wk_cv_prog_COMPHEP_F77_cross)
AC_LANG_RESTORE
])
if test $wk_cv_prog_COMPHEP_F77_works = no; then
  AC_MSG_ERROR([installation or configuration problem: f77 compiler cannot create executables.
Check $comphepcom/F77 if it is set up correctly])
fi
AC_CACHE_CHECK([whether this compiler is a cross-compiler],
[wk_cv_prog_COMPHEP_F77_cross],
[cross_compiling=$wk_cv_prog_COMPHEP_F77_cross
])])



dnl WK_LANG_COMPHEP_CC()
AC_DEFUN(WK_LANG_COMPHEP_CC,
[define([AC_LANG], [COMPHEP_CC])dnl
ac_ext=c
ac_compile='sh $comphepcom/CC conftest.$ac_ext 1>&AC_FD_CC'
ac_link='sh $comphepcom/CC -o conftest${ac_exeext} conftest.$ac_ext $LIBS 1>&AC_FD_CC'
cross_compiling=$wk_cv_prog_COMPHEP_CC_cross
])


dnl WK_LANG_COMPHEP_F77()
AC_DEFUN(WK_LANG_COMPHEP_F77,
[define([AC_LANG], [COMPHEP_F77])dnl
ac_ext=f
ac_compile='sh $comphepcom/F77 conftest.$ac_ext 1>&AC_FD_CC'
ac_link='sh $comphepcom/F77 -o conftest${ac_exeext} conftest.$ac_ext $LIBS 1>&AC_FD_CC'
cross_compiling=$wk_cv_prog_COMPHEP_F77_cross
])



dnl Check for the MPI Fortran 90 compiler. 
dnl Set the output variable `MPIF90' to the name of the compiler found.
dnl If no compiler found, set `MPIF90' to the value of `F90'
dnl 
dnl WK_PROG_MPIF90()
AC_DEFUN(WK_PROG_MPIF90,
[AC_BEFORE([$0], [AC_PROG_CPP])dnl
if test -z "$MPIF90"; then
  AC_PATH_PROGS(MPIF90, mpif95 mpif90)
  test -z "$MPIF90" && AC_MSG_WARN([*** No MPI Fortran 90 compiler found in \$PATH.
Parallel version won't be available.])
fi
if test -n "$MPIF90"; then
  WK_PROG_MPIF90_WORKS
else
  MPIF90=$F90
fi
])

dnl Test whether the MPI Fortran 90 compiler can compile and link a trivial
dnl Fortran program.  Also, test whether the Fortran 90 compiler is a
dnl cross-compiler.
dnl 
dnl WK_PROG_MPIF90_WORKS()
AC_DEFUN(WK_PROG_MPIF90_WORKS,
[AC_MSG_CHECKING([whether the MPI Fortran 90 compiler ($MPIF90 $F90FLAGS $LDFLAGS) works])
AC_LANG_SAVE
WK_LANG_FORTRAN90
WK_TRY_COMPILER_NOINCLUDE(dnl
[program conftest
end program conftest
], wk_cv_prog_mpif90_works, wk_cv_prog_mpif90_cross)
AC_LANG_RESTORE
AC_MSG_RESULT($wk_cv_prog_mpif90_works)

if test $wk_cv_prog_f90_works = no; then
  AC_MSG_ERROR([installation or configuration problem: MPI Fortran 90 compiler cannot create executables.])
fi
])


dnl *** This is probably obsolete:
dnl Check for the f77 runtime library.  This may be needed for linking 
dnl f90 programs with f77 libraries, at least on gcc-based Linux.
dnl We use the LD_LIBRARY_PATH.  Since we are on Linux,
dnl we may try also to find it with 'locate'.
dnl The path part of the result is stored in F77_LIBDIR
dnl If it is not found, return 'no'.  If we are not on Linux, leave this empty.
dnl
dnl WK_CHECK_F77_RUNTIME_LIBRARY()
AC_DEFUN(WK_CHECK_F77_RUNTIME_LIBRARY,
[if test "`uname`" = "Linux"
then
AC_CHECK_PROG(LOCATE,locate,yes)
if test "$LOCATE" = "yes"
then
LD_LIBRARY_PATH=`locate libg2c.a | sed -e 's!/libg2c.a!!' | xargs echo | sed -e 's/ /:/g'`:$LD_LIBRARY_PATH
fi
AC_PATH_PROG(F77_RUNTIME_LIBRARY, libg2c.a, no, $LD_LIBRARY_PATH)
F77_LIBDIR=`echo $F77_RUNTIME_LIBRARY | sed -e s!/libg2c.a!!`
AC_SUBST(F77_LIBDIR)
fi
])


dnl Test whether Metapost works and can dump a .mem file
dnl
dnl WK_PROG_MPOST_WORKS()
AC_DEFUN(WK_PROG_MPOST_WORKS,
[AC_CACHE_CHECK([whether Metapost ($MPOST) works], [wk_cv_prog_mpost_works],
[touch conftest.mp
echo dump | $MPOST --ini conftest >/dev/null 2>&1
if test -f conftest.mem; then
  wk_cv_prog_mpost_works="yes"
else
  wk_cv_prog_mpost_works="no"
fi
rm -f conftest.mp conftest.mem conftest.log

if test $wk_cv_prog_mpost_works = "no"; then
  AC_MSG_ERROR([installation or configuration problem: METAPOST cannot create .mem files.])
fi
dnl AC_MSG_RESULT($wk_cv_prog_mpost_works)
])])


dnl Test whether PYTHIA supports the new interface (versions from 6.2 on)
dnl
dnl WK_CHECK_PYTHIA_INTERFACE
AC_DEFUN(WK_CHECK_PYTHIA_INTERFACE,
[AC_CACHE_CHECK([pythia for the Les Houches user process standard], 
[wk_cv_pythia_supports_heprup],
[if (ar t $PYTHIA | grep upinit > /dev/null); then
  wk_cv_pythia_supports_heprup="yes"
else
  wk_cv_pythia_supports_heprup="no"
fi
])])
